@@ -4,17 +4,16 @@
 #' Prior objects for bssm models
 #'
 #' These simple objects of class \code{bssm_prior} are used to construct a prior distributions for the 
-#' MCMC runs of \code{bssm} package. Currently supported priors are uniform, half-Normal and Normal distribution.
+#' MCMC runs of \code{bssm} package. Currently supported priors are uniform (\code{uniform()}), 
+#' half-normal (\code{halfnormal()}), normal (\code{normal()}), and truncated normal distribution  (\code{tnormal()}).
 #' 
-#' @note Use Normal prior with care: currently there is no checks of non-negativity of standard deviation or stationarity of 
-#' autoregressive coefficient in case Normal prior is used.
 #' 
 #' @rdname priors
 #' @param init Initial value for the parameter, used in initializing the model components and as a starting value
 #' in MCMC.
-#' @param min Lower bound of the uniform prior .
-#' @param max Upper bound of the uniform prior.
-#' @param sd Standard deviation of the half-Normal and Normal priors.
+#' @param min Lower bound of the uniform and truncated normal prior.
+#' @param max Upper bound of the uniform and truncated normal prior.
+#' @param sd Standard deviation of the (underlying i.e. non-truncated) Normal distribution.
 #' @param mean Mean of the Normal prior.
 #' @return object of class \code{bssm_prior}.
 #' @export
@@ -85,18 +84,39 @@ normal <- function(init, mean, sd){
       class = "bssm_prior")
   }
 }
-
+#' @rdname priors
+#' @export
+tnormal <- function(init, mean, sd, min = -Inf, max = Inf){
+  
+  if(any(!is.numeric(init), !is.numeric(mean), !is.numeric(sd))) {
+    stop("Parameters for priors must be numeric.")
+  }
+  if (any(sd < 0)) {
+    stop("Standard deviation parameter for Normal distribution must be positive.")
+  }
+  
+  n <- max(length(init), length(mean), length(sd))
+  if (n > 1) {
+    structure(lapply(1:n, function(i) structure(list(prior_distribution = "tnormal", 
+      init = safe_pick(init, i), mean = safe_pick(mean, i), sd = safe_pick(sd, i),
+      min = safe_pick(min, i), max = safe_pick(max, i)), 
+      class = "bssm_prior")), class = "bssm_prior_list")
+  } else {
+    structure(list(prior_distribution = "tnormal", init = init, mean = mean, sd = sd, 
+              min = min, max = max), class = "bssm_prior")
+  }
+}
 combine_priors <- function(x) {
   
   if (length(x) == 0) return(list(prior_distributions = 0, parameters = matrix(0, 0, 0)))
   
   prior_distributions <- sapply(x, "[[", "prior_distribution")
-  parameters <- matrix(NA, 2, length(prior_distributions))
+  parameters <- matrix(NA, 4, length(prior_distributions))
   for(i in 1:length(prior_distributions)) {
     parameters[1:(length(x[[i]])-2), i] <- as.numeric(x[[i]][-(1:2)])
   }
   list(prior_distributions = 
-      pmatch(prior_distributions, c("uniform", "halfnormal", "normal"), duplicates.ok = TRUE)-1, 
+      pmatch(prior_distributions, c("uniform", "halfnormal", "normal", "tnormal"), duplicates.ok = TRUE)-1, 
     parameters = parameters)
 }
 
